#
# Powershell script to submit jobs to the Hercules reader
# Usage: hercrdr.ps1 [-server hostname] [-port number] [-jcl filename] [-user mvsname] [-pass mvspass]
#

param (
	[switch]$pipe = $false,
	[string]$server = "localhost",
	[string]$port = "3505",
	[string]$jcl = "",
	[string]$user = "",
	[string]$pass = ""
)

$cwd = Get-Location
Write-Host "Current Directory: ${cwd}"

# we're in interactive mode if we've not been given a JCL file on the command line
$interactive = [string]::IsNullOrEmpty($jcl)

if ($interactive) {	
	$jcl = Read-Host "Enter jcl file name (enter to exit) "
}

while ( ! [string]::IsNullOrEmpty($jcl)) {
	try {
		$fname = $jcl -replace '"', '' # strip any quotes !
		$data = Get-Content -Path $fname  -ErrorAction Stop
	} catch {
		Write-Host "Could not open jcl file: ${jcl}"
		Write-Host ""
		$jcl = Read-Host "Enter jcl file name (enter to exit) "
		continue
	}
	#
	# if the jcl text has %user% or %pass% included then request, if not set, and modify the cards before submission
	#
	if ( $data -match "%user%") {
		if ([string]::IsNullOrEmpty($user)) {
			$user = Read-Host "Enter MVS username "
		}
		$data = $data -replace "%user%", ${user}.toupper()
	}
	if ( $data -match "%pass%") {
		if ([string]::IsNullOrEmpty($pass)) {
			$pass = Read-Host "Enter MVS password "
		}
		$data = $data -replace "%pass%", ${pass}.toupper()
	}
	#
	# send cards to reader
	#
	try {
		$socket = new-object System.Net.Sockets.TcpClient($server, $port)
		$stream = $socket.GetStream()
	} catch {
		Write-Host "Fatal error: Cannot open Reader at ${server}:${port}"  -ForegroundColor red
		break
	}
	Write-Host "Connected to Reader on ${server} port ${port}"
	$n = 0				# number of cards sent
	foreach ($line in $data)
	{
		$line = $line + "`n"
		$card = [System.Text.Encoding]::ASCII.GetBytes($line)
		try {
			$stream.Write($card, 0, $card.Length)
		} catch {
			Write-Host "Write to ${server}:${port} failed at card ${n}"  -ForegroundColor red
			break
		}
		$n++
	} 
	$stream.Close()
	Write-Host "${n} cards submitted from ${jcl} to ${server}:${port}"
	Write-Host ""
	$jcl = Read-Host "Enter jcl file name "
} 

if ( $interactive ){
	Read-Host "Hit any key to exit...."
}
