//* Jobstream to assemble and demonstrate a small Assembler routine that
//* allows a PL/1(F) program to make a call to a COBOL sub-program (or
//* to any language that obeys the standard 360/370 argument-passing
//* conventions).
//*
//* The Assembler routine is named ASMDEMOZ. It has two entry points,
//* one for passing arguments of alphanumeric data types, and the other
//* for passing linear arguments such as binary and packed data types.
//* 
//* A calling PL/1(F) program should call whichever entry point is
//* appropriate for the data types being passed to COBOL - or both,
//* if appropriate, as is demonstrated by this jobstream.
//* 
//* The jobstream includes a PL/1(F) MAIN program and a COBOL
//* sub-program which between them demonstrate the use of the
//* Assembler interface. PL/1(F) passes some values to the COBOL program,
//* which multiplies the values together and passes the result back to 
//* PL/1(F) for displaying on the console. That's it, nothing fancy (or
//* even useful).
//* 
//* Please contact me with any comments, queries, improvements, etc.
//*
//* John James April 2020. In lockdown!
//* johnjamesmain@gmail.com
//*
//*
//*
//PL1FDEMO JOB      (01),'PL1(F) COBOL CALL',
//         REGION=2048K,USER=HERC01,PASSWORD=CUL8TR  
//*
//S1       EXEC     PGM=IDCAMS
//SYSPRINT DD       SYSOUT=*
//SYSIN    DD       *
  DELETE HERC01.TEST.LOADLIB(COBDEMOZ)
  DELETE HERC01.TEST.LOADLIB(ASMDEMOZ)
  DELETE HERC01.TEST.LOADLIB(PL1DEMOZ)
//S2       EXEC     COMPRESS,LIB='HERC01.TEST.LOADLIB'
//S3       EXEC     COBUCL
//COB.SYSIN  DD     *
000100 IDENTIFICATION DIVISION.
000200 PROGRAM-ID. 'COBDEMOZ'.
000300 ENVIRONMENT DIVISION.
000400 CONFIGURATION SECTION.
000500 SOURCE-COMPUTER. IBM-360.
000600 OBJECT-COMPUTER. IBM-360.
000900 DATA DIVISION.
001000 WORKING-STORAGE SECTION.
001100     77 CONST01  PIC X(28) VALUE '>>>> THE NUMBER IS POSITIVE '.
001110     77 CONST02  PIC X(28) VALUE '>>>> THE NUMBER IS NEGATIVE '.
001115     77 CONST03  PIC X(28) VALUE '>>>> THE NUMBER IS ZERO     '.
001118     77 TEMP1    PIC S9(5).
001120 LINKAGE SECTION.
001130     01 ARG1C    PIC X(28).
001140     01 ARG2C    PIC 9(5).
001160     01 ARG1     PIC S9(5) COMP.
001170     01 ARG2     PIC S9(5) COMP-3.
001200 PROCEDURE DIVISION.
001300     ENTRY 'COBENT0' USING ARG1C, ARG2C.
001310     GOBACK.
001320     ENTRY 'COBENT1' USING ARG1, ARG2.
001400     MULTIPLY ARG1 BY ARG2 GIVING TEMP1.
001405     IF TEMP1 EQUAL TO ZERO THEN MOVE CONST03 TO ARG1C.
001410     IF TEMP1 IS GREATER THAN ZERO THEN MOVE CONST01 TO ARG1C.
001430     IF TEMP1 IS LESS THAN ZERO THEN MOVE CONST02 TO ARG1C.
001500     MOVE TEMP1 TO ARG2C.
001600     GOBACK.
/*
//LKED.SYSLMOD DD DSN=HERC01.TEST.LOADLIB(COBDEMOZ),DISP=SHR
//S4       EXEC     ASMFCL
//ASM.SYSIN  DD     *
**********************************************************************
*
*    PL1(F) TO COBOL INTERFACE
*
*    JOHN JAMES 2015-2019
*
*    ALLOWS A PL1(F) PROGRAM TO INVOKE A COBOL SUB-PROGRAM.
*
*    2 INTERFACES ARE PROVIDED, ONE AT COBITF0 FOR ALPHANUMERIC DATA
*    TYPES, WHICH INVOKES COBOL AT ENTRY POINT COBENT0;
*    THE OTHER AT COBITF1 FOR BINARY AND PACKED DATA TYPES, WHICH
*    INVOKES COBOL AT ENTRY POINT COBENT1.
*
*
*
*
*                FIRST OF 2 INTERFACES PL/1 TO COBOL
*
COBITF0  CSECT
*
* PL/1 TO COBOL INTERFACE CORRECTION FOR CHARACTER ARGUMENTS
*
         USING *,15
         B     P1              BRANCH AROUND CONSTANTS
         DC    AL1(7)          NAME LENGTH
         DC    C'COBITF0'      NAME
P1       STM   14,11,12(13)    LEAVE R12 IN SAVEAREA
         DROP  15
         BALR  10,0            ESTABLISH NEW ADDRESSABILITY
         USING *,10
         LA    0,100           REQUIRED BY IHESADA
         L     15,=V(IHESADA)  CALL PL/1 GET DSA ROUTINE
         BALR  14,15
         MVI   0(13),X'80'     MOVE IN FLAG BYTE
*
*                              COMPLETE SAVEAREA CHAINING
*
         LA    15,SAVEARE
         ST    15,8(13)
         ST    13,4(15)
         LR    13,15
*
         LTR   1,1             DOPE VECTOR TABLE POINTER ZERO?
         BZ    NOPARM2         BRANCH IF SO
         LA    3,OUTPARM       R3 POINTS TO NEXT OUTPUT POINTER
         LA    5,PARMEND       POINT TO MAX EXTENT OF OUTPUT ARGUMENTS
         B     MOVEPARM        GO COPY THE OUTPUT ARGUMENT POINTERS
*
NEXTPARM LA    1,4(1)          ADDRESS OF NEXT DOPE VECTOR POINTER
         LA    3,4(3)          NEXT OUTPUT ARGUMENT POINTER
MOVEPARM L     4,0(1)          POINT TO NEXT DOPE VECTOR
         L     2,0(4)          GET ADDRESS FROM THAT POINTER
         CR    3,5             TOO MANY ARGUMENTS?
         BNL   TOOMANY         ABEND IF SO
         ST    2,0(3)          ELSE PUT ARGUMENT ADDRESS IN OUTPUT LIST
         TM    0(1),X'80'      LOOP BACK IF MORE POINTERS
         BZ    NEXTPARM
*
         OI    0(3),X'80'      AT END, INDICATE LAST POINTER
         LA    1,OUTPARM       POINT R1 TO NEW LIST
         B     CALLSUB2        GO MAKE THE CALL
*
NOPARM2  XR    1,1             HANDLE NO ARGUMENTS SITUATION
*
CALLSUB2 L     15,=V(COBENT0)
         BALR  14,15           CALL COBOL SUBROUTINE
*
*                              ON RETURN FROM SUBROUTINE...
         L     13,SAVEARE1+4   RESTORE CALLER'S ENVIRONMENT
         LM    14,11,12(13)
         DROP  10
         BALR  15,0            R10 DESTROYED, NEW ADDRESSABILITY
         USING *,15
         L     15,=V(IHESAFA)  TELL PL/1 TO RETURN TO CALLER
         BR    15
*
TOOMANY  WTO   'COBITF0 ERROR - TERMINATING'
         ABEND 0016
*
SAVEARE1 DS    18F
OUTPARM  DS    16F
PARMEND  EQU   *
         LTORG
         DROP  15
*
* SECOND PL/1 TO COBOL INTERFACE FOR LINEAR ARGUMENTS
*
COBITF1  CSECT
         USING *,15
         B     P2              BRANCH AROUND CONSTANTS
         DC    AL1(7)          NAME LENGTH
         DC    C'COBITF1'      NAME
P2       STM   14,11,12(13)    LEAVE R12 IN SAVEAREA
         DROP  15
         BALR  10,0            ESTABLISH NEW ADDRESSABILITY
         USING *,10
         LA    0,100           REQUIRED BY IHESADA
         L     15,=V(IHESADA)  CALL PL/1 GET DSA ROUTINE
         BALR  14,15
         MVI   0(13),X'80'     MOVE IN FLAG BYTE
*
*                              COMPLETE SAVEAREA CHAINING
*
         LA    15,SAVEAREB
         ST    15,8(13)
         ST    13,4(15)
         LR    13,15
*
         L     15,=V(COBENT1)
         BALR  14,15           CALL COBOL SUBROUTINE
*
*                              ON RETURN FROM SUBROUTINE...
         L     13,SAVEAREB+4   RESTORE CALLER'S ENVIRONMENT
         LM    14,11,12(13)
         DROP  10
         BALR  15,0            R10 DESTROYED, NEW ADDRESSABILITY
         USING *,15
         L     15,=V(IHESAFA)  TELL PL/1 TO RETURN TO CALLER
         BR    15
*
*
SAVEAREB DS    18F
         LTORG
         END
/*
//LKED.SYSLMOD DD DSN=HERC01.TEST.LOADLIB(ASMDEMOZ),DISP=SHR
//S5       EXEC    PL1LFCLG
//PL1L.SYSIN   DD *
 SAMP0: PROC OPTIONS(MAIN);
 DCL XC1     CHARACTER(28);
 DCL XC2     CHARACTER(5);
 DCL XN1     FIXED BINARY(31,0);
 DCL XN2     FIXED DECIMAL(5);
 DCL COBITF0 ENTRY (         CHARACTER (28),
                             CHARACTER(5));
 DCL COBITF1 ENTRY (         FIXED BINARY(31,0),
                             FIXED DECIMAL(5));
 DISPLAY ('>>>> DEMO STARTED');
 DISPLAY ('>>>> GENERATE SOME POSITIVE PRODUCTS');
 XN1 = 123;
 XN2 = 4;
 CALL COBITF0 (XC1, XC2);
 CALL COBITF1 (XN1, XN2);
 DISPLAY (XC1); DISPLAY(XC2);
 XN1 = 500;
 XN2 = 3;
 CALL COBITF0 (XC1, XC2);
 CALL COBITF1 (XN1, XN2);
 DISPLAY (XC1); DISPLAY(XC2);
 XN1 = 9;
 XN2 = 100;
 CALL COBITF0 (XC1, XC2);
 CALL COBITF1 (XN1, XN2);
 DISPLAY (XC1); DISPLAY(XC2);
 DISPLAY ('>>>> GENERATE A ZERO RESULT');
 XN1 = 123;
 XN2 = 0;
 CALL COBITF0 (XC1, XC2);
 CALL COBITF1 (XN1, XN2);
 DISPLAY (XC1); DISPLAY(XC2);
 DISPLAY ('>>>> GENERATE A NEGATIVE PRODUCT');
 XN1 = -123;
 XN2 = 10;
 CALL COBITF0 (XC1, XC2);
 CALL COBITF1 (XN1, XN2);
 DISPLAY (XC1); DISPLAY(XC2);
 DISPLAY ('>>>> DEMO ENDING');
 END SAMP0;
/*
//LKED.SYSLMOD DD DSN=HERC01.TEST.LOADLIB(PL1DEMOZ),DISP=SHR
//LKED.HERCLIB DD DSN=HERC01.TEST.LOADLIB,DISP=SHR
//LKED.SYSIN   DD *
   INCLUDE HERCLIB(COBDEMOZ,ASMDEMOZ)
/*
//