/* Copyright (c) 2012: Daniel Richman. License: GNU GPL 3 */
/* Additional features: Priyesh Patel                     */
/* Added limit display to number of lines capability: Andrew Gadsby */

(function () {

var dataelem = "#data";
var pausetoggle = "#pause";
var gotoend = "#GotoEnd";
var pollarea = "#poll";

var url = "getconsole.php";		/* log scrapping program MUST understand html range */
var fix_rn = false;			/* convert \n to \r\n */
var max_lines = 1000;			/* limit to last N lines in log */
var console_lines = 25;			/* number to display */
var load = max_lines * ( 80 + 2 ); 	/* buffer for lines, max 80 long + <cr><nl> */
var def_lines = 24;			/* default lines to display */
var poll = 5; 				/* secs */

var kill = false;
var loading = false;
var pause = false;
var log_data = "";
var log_file_size = 0;
var cmd_buffer = [];			/* used for command recall */
var cmd_pos = 0;

function get_log() {
    if (kill | loading) return;
    loading = true;

    var range;
    var first_load;
    var must_get_206;
    if (log_file_size === 0) {
        /* Get the last 'load' bytes */
        range = "-" + load.toString();
        first_load = true;
        must_get_206 = false;
    } else {
        /* Get the (log_file_size - 1)th byte, onwards. */
        range = (log_file_size - 1).toString() + "-";
        first_load = false;
        must_get_206 = log_file_size > 1;
    }

    /* The "log_file_size - 1" deliberately reloads the last byte, which we already
     * have. This is to prevent a 416 "Range unsatisfiable" error: a response
     * of length 1 tells us that the file hasn't changed yet. A 416 shows that
     * the file has been truncated */

    $.ajax(url, {
        dataType: "text",
        cache: false,
        headers: {Range: "bytes=" + range},
        success: function (data, s, xhr) {
            loading = false;

            var content_size;
	    var new_file_size;

            if (xhr.status === 206) {
                var c_r = xhr.getResponseHeader("Content-Range");
                if (!c_r)
                    throw "Server did not respond with a Content-Range";

                new_file_size = parseInt(c_r.split("/")[1], 10);
		if (new_file_size < log_file_size) {
			/* file truncated so reset everything 
			 * we will show an empty screen for one cycle and 
			 * the tail on the next
			 */
			first_load = true;
			log_file_size = 0;
		} else
			log_file_size = new_file_size;
                content_size = parseInt(xhr.getResponseHeader("Content-Length"), 10);

            } else if (xhr.status === 200) {
                if (must_get_206)
                    throw "Expected 206 Partial Content";

                content_size = log_file_size =
                        parseInt(xhr.getResponseHeader("Content-Length"), 10);
            } else {
                throw "Unexpected status " + xhr.status;
            }

            if (first_load && data.length > load)
                throw "Server's response was too long";

            var added = false;

            if (first_load) {
                /* Clip leading part-line if not the whole file */
                if (content_size < log_file_size) {
                    var start = data.indexOf("\n");
                    log_data = data.substring(start + 1);
                } else {
                    log_data = data;
                }

                added = true;
            } else {
                /* Drop the first byte (see above) */
                log_data += data.substring(1);

                if (log_data.length > load) {
                    var start = log_data.indexOf("\n", log_data.length - load);
                    log_data = log_data.substring(start + 1);
                }

                if (data.length > 1)
                    added = true;
            }

            if (added)
                show_log();
            setTimeout(get_log, poll * 1000);
        },
        error: function (xhr, s, t) {
            loading = false;

            if (xhr.status === 416 ) {
                /* 416: Requested range not satisfiable: log was truncated. */
                log_file_size = 0;
                log_data = "";
                show_log();

                setTimeout(get_log, poll * 1000);
            } else if (xhr.status === 404) {
		/* 404 is file not found */
		throw "Host console log not found";
	    } else {
                throw "AJAX Error (status " + xhr.status + ")";
            }
        }
    });
}

function scroll(where) {
	var out = document.getElementById("data");
	    // if where = -1 scroll to bottom unless user scrolled up.
	//var isScrolledToBottom = (out.scrollHeight - out.clientHeight) <= (out.scrollTop + 10);
	console.log( out.scrollHeight, out.clientHeight, out.scrollTop);
        if (where === -1) 
	    out.scrollTop = out.scrollHeight - out.clientHeight;	
        else
            out.scrollTop = where;
}

/* get ONLY last N lines from retrieved data */
function getlastNlines(s, n) {
	if (n <= 0)			/* special case return everything */
		return s;
	
	var t = s.split("\n");		/* get array of lines */
	
	var numtoremove = t.length - n;	/* get rid of oldest lines */
	for (var i =  0; i < numtoremove ; i++)
		t.shift();		/* remove oldest line */
	return t.join("\n");		/* glue back together */
}


function show_log() {
    if (pause) return;

    var t = log_data;
    var lines = def_lines;	/* default console window size */
    
    if (document.getElementById('lines'))
	lines = document.getElementById('lines').value;
	
    if (isNaN(lines) || lines <= 0 )
	lines = def_lines;
    else if (lines > max_lines)
	lines = max_lines;

    t = getlastNlines(t, lines);

    if (fix_rn)
        t = t.replace(/\n/g, "\r\n");

    $(dataelem).text(t);	// display lines

    scroll(-1);			// scroll to end unless user paused display

}

function error(what) {
    kill = true;

    $(dataelem).text("An error occured => " + what + "\r\n" +
                     "Reloading this page may help.\r\n");
    scroll(0);

    return false;
}

// command recall on up and down arrow
$(document).on("keydown", '#cmd', function (e) {
    if (e.keyCode == 38) 	// up arrow
        $("#cmd").val(cmd_pos < cmd_buffer.length ? cmd_buffer[cmd_pos++] : "");
    if (e.keyCode == 40) 	// down arrow
	$("#cmd").val(cmd_pos > 0 ? cmd_buffer[--cmd_pos] : "");
});

$(document).ready(function () {
    window.onerror = error;
	
    /* Add pause toggle */
    $(pausetoggle).click(function (e) {
        pause = !pause;
        $(pausetoggle).text(pause ? "Continue" : "Pause");
        show_log();
        e.preventDefault();
    });
    
    /* Add Goto End of Log button */
    $(gotoend).click(function (e) {
	show_log();
	scroll(-1);
        e.preventDefault();
    });
    
    /* update interval */
    $(pollarea).change(function(e) {
		var newpoll;
		e.preventDefault();

		newpoll = Number($(pollarea).val());
		if (isNaN(newpoll) || newpoll < 1)
			alert("Polling interval must be a number > 0");
		else
			poll = newpoll;
		$(pollarea).val(poll);
    });

    $("#send").click(function(e) {
      // send command to input function for MVS/JES to handle
      e.preventDefault();		// prevent refresh of whole page
      var cmd = $("#cmd").val();   
      var dataString = 'cmd=' + encodeURIComponent(cmd);
       $.ajax({
	type: "POST",
	data: dataString,
	dataType: "text",
	success: function (data, s, xhr) {
		$("#rdrmsg").text("OK "+ cmd);
		$("#cmd").focus();
		},
	error: function (xhr, s, t) {
		$("#rdrmsg").text("Error on '" + cmd +"' : " + t + " " + xhr.status);
		$("#cmd").focus();
		}
	});
	$("#rdrmsg").text("SENT "+ cmd);
	$("#cmd").val("");		// clear to prevent accidental re-entry
	cmd_buffer.unshift(cmd);	// save for recall
	cmd_pos = 0;

    });

    get_log();
});

})();