<?php
require ("config.php");

$MVSControl = array(			// single cookie holding useful state (json encoded)
	"system" => key($sites),	// current site being viewed
	"style" => "icon", 		// print job summary page layout 
	"order" => "desc",		// print job sort order
	"age" 	=> 0,			// print jobs age to show
	"pagesize" => "a4",		// print job paper type
	"colour" => array( "default" => "green"),
);

$cookies = getAllCookieJson();
$update = 0;

foreach ($MVSControl as $c => $def) {
	if ( $cookies && isset($cookies[$c]))
		$MVSControl[$c] = $cookies[$c];
	if (isset($_REQUEST[$c])) {			// form setting overides
		if ($c == "colour")
		{
			// special case as colour build up hash array where key = "/class/type"
			list($class, $type, $job) = getRequest();
			$MVSControl[$c]["/$class/$type"] = $_REQUEST[$c];
		
		} else
			$MVSControl[$c] = $_REQUEST[$c];
		$update++;
	}
	if ($update)
		setCookieJson($MVSControl);
}

$cursite = $MVSControl["system"];

if (isset($sites[$cursite]) === false) {
	// system cookie may have been spoofed or site removed so default to first in config.php
	$cursite = key($sites);
}

$CONSOLE_PATH = $sites[$cursite]["CONSOLE"];
$PRT_DIR = 	$sites[$cursite]["PRTDIR"];
$JCL_DIR = 	$sites[$cursite]["JCLDIR"];
$HERC_RDR_SOCK = $sites[$cursite]["READER"];
$CONS_INPUT = 	$sites[$cursite]["HERCCONS"];
$TAPE_DIR = 	$sites[$cursite]["TAPEDIR"];
$IPL_CMD =	$sites[$cursite]["IPLCMD"];
$FONT = 	$sites[$cursite]["FONT"];


function check_auth_ok($required = "")
{
	if ( ! SECURED ) {
		return true;
	}
	if (session_id() === "")
		session_start();
	if ($required === "") {
		return true;
	}
	if(!isset($_SESSION['session_user'])){
		return false;
	}
	if (isset($_SESSION[$required]) && $_SESSION[$required] == true) {
		return true;
	}
	return false;	
}

function do_header($msg) 
{
	global $cursite;
	
	echo "<header><h2>$msg: ";
	echo $cursite;
	echo "</h2></header>";
}

function do_footer($msg = "") 
{
}

// show navigation links to other pages
function show_nav($tab = "", $prefix=".")
{
	echo '<a href="help.html">Help</a><br><br>';
	if ($tab !== "admin" && SECURED && check_auth_ok("admin")) echo '<a href="'.$prefix.'/admin/index.php" target="auth">Admin</a><br>'; 
	if ($tab !== "console" && check_auth_ok("console")) echo '<a href="'.$prefix.'/console/index.php" target="console">Console</a><br>';
	if ($tab !== "reader" && check_auth_ok("reader"))   echo '<a href="'.$prefix.'/reader/index.php" target="rdr">Card Reader</a><br>';
	if ($tab !== "printer" && check_auth_ok("output"))  echo '<a href="'.$prefix.'/printer/index.php" target="printer">Output Control</a><br>';
	if ($tab !== "tape" && check_auth_ok("tape"))       echo '<a href="'.$prefix.'/tape/index.php"  target="tape">Tape Management</a><br>';

	if ($tab === "" && SECURED && isset($_SESSION['session_user']))
		echo '<br><a href="admin/password.php">Change Password</a><br>';
}

/**
 * Sets and fills the cookie 'mvs_jar' with json name:value
 * If the value is empty, it deletes the name:val pair
 *
 * @param $array - name => value
 *               We use an array because of multiple settings on one page,
 *               e.g. options from radiobuttons after the submit
 * @return string - value
 *
 * based on example from www.amalesh.de/2016/04/18/how-to-save-namevalue-pairs-as-json-in-one-cookie/ 
 */
function setCookieJson($arr) {

      // the cookie which saves the json
      if (isset($_COOKIE['mvs_jar'])) {
         // read the json
         $data = json_decode($_COOKIE['mvs_jar'],true);

         // to delete unwanted
	 if ($data) {
		//foreach ($data as $index => $item) {
		//    if(isset($arr[$index]) && $arr[$index] === '') {
		//	unset($data[$index],$arr[$index]);
		//    }
		//}
		// merge them
		$arr = array_merge($data,$arr);
	 }
      }
      setcookie('mvs_jar',json_encode($arr),time() + 31536000, "/"); // one year
}

/**
 * Get the value from the json-cookie
 * If not set returns the default value passed in
 *
 * @param $name - element name
 * @return mixed - the value
 */
function getCookieJson($name, $def) {

   // read the container cookie if set
   if (isset($_COOKIE['mvs_jar'])) {
	$data = json_decode($_COOKIE['mvs_jar'],true);

	if($data && isset($data[$name])) 
		return $data[$name];
   }
   // nothing found
   return $def;
}

/**
 * return all cookies that have been set as an array of key value pairs
 */
 function getAllCookieJson() {

    // the cookie which saves the json
    if (isset($_COOKIE['mvs_jar'])) {
        // read the json
        return json_decode($_COOKIE['mvs_jar'],true);
    } else
	return null;
 }
 
/*
 * extract and validate job information
 */
 function getRequest()
 {
	$class = $type = $job = null;
	
	if (isset($_REQUEST["class"]) && (preg_match("/^[A-Z0-9]$/", $_REQUEST["class"]) || $_REQUEST["class"] == "LOG") )
		$class = $_REQUEST["class"];
		
	if (isset($_REQUEST["type"]) && preg_match("/^[A-Z0-9][A-Z0-9][A-Z0-9]$/", $_REQUEST["type"]))
		$type =  $_REQUEST["type"];

	if (isset($_REQUEST["job"]) && preg_match("/^[0-9]*_[0-9]*\.lst$/", $_REQUEST["job"]))
		$job = $_REQUEST["job"];
	
	return array($class, $type, $job);
 }

function caching_headers ($name, $timestamp) {
    $gmt_mtime = gmdate('r', $timestamp);
    header('ETag: "'.md5($timestamp.$name).'"');
    header('Last-Modified: '.$gmt_mtime);
    header('Cache-Control: public');

    if(isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) || isset($_SERVER['HTTP_IF_NONE_MATCH'])) {
        if ($_SERVER['HTTP_IF_MODIFIED_SINCE'] == $gmt_mtime || str_replace('"', '', stripslashes($_SERVER['HTTP_IF_NONE_MATCH'])) == md5($timestamp.$name)) {
            header('HTTP/1.1 304 Not Modified');
            exit();
        }
    }
}

?>
