<?php 
/**
 * Sets and fills the cookie 'container' with json name:value
 * If the value is empty, it deletes the name:val pair
 *
 * @param $array - name => value
 *               We use an array because of multiple settings on one page,
 *               e.g. options from radiobuttons after the submit
 * @return string - value
 *
 * from www.amalesh.de/2016/04/18/how-to-save-namevalue-pairs-as-json-in-one-cookie/ 
 */
function setCookieJson($arr) {

      // the cookie which saves the json
      if ($_COOKIE['container']) {
         // read the json
         $data = json_decode($_COOKIE['container'],true);

         // to delete unwanted
         foreach ($data as $index => $item) {
            if(isset($arr[$index]) && $arr[$index] === '') {
		unset($data[$index],$arr[$index]);
            }
         }
         // merge them
         $arr = array_merge($data,$arr);
      }
      setcookie('container',json_encode($arr),time() + 31536000); // one year
   }
}

/**
 * Get the value from the json-cookie
 * If not set returns the default value passed in
 *
 * @param $name - element name
 * @return mixed - the value
 */
function getCookieJson($name, $def) {

   // read the container cookie
   $data = json_decode($_COOKIE['container'],true);

   if(isset($data[$name])) {
      return $data[$name];
   }
   // nothing found
   return $def;
}

/**
 * return all cookies that have been set as an array of key value pairs
 */
 function setAllCookieJson($arr) {

    // the cookie which saves the json
    if ($_COOKIE['container']) {
        // read the json
        return json_decode($_COOKIE['container'],true);
    } else
	return null;
 }
 

// examples
$my_cookie['foo'] = 'bar';
$my_cookie['foobar'] = 'foobar';

// set
setCookieJson($my_cookie); // {"foo":"bar","foobar":"foobar"}

// get
$cookie_foo = getCookieJson('foo'); // bar

// delete
$my_cookie['foobar'] = '';
setCookieJson($my_cookie); // {"foo":"bar"}
?>