<?php 
// class to handle navigating the output print directory structure
class jobOutput {
	
	protected $root;
	protected $oTree;
	protected $JobOrderAscending;
		
	function __construct($root) 
	{
		$this->root = $root;	// top level / classes / jobs
		$this->JobOrderAscending = false;
		$this->oTree = $this->getClassTypeTree();
	}
	function sortAscending()
	{
		$this->JobOrderAscending = true;
	}
	// traverse the output directory tree to get the classes, type and jobs in an associative array [class][type]jobs
	function getClassTypeTree()
	{
		if (isset($this->oTree))
			return $this->oTree;
		$this->oTree = $this->traverseTree($this->root, '/[A-Z0-9]/');
		$this->sortOutputTree($this->oTree);
		return $this->oTree;
	}
	
	protected function traverseTree($path, $match)
	{
	    
	    $first = true;
	    
	    if(is_dir($path))
	    {
		if($handle = opendir($path))
		{
		    while(($name = readdir($handle)) !== false)
		    {
			if( $name != "." && $name != "..") 
			{

				if(is_dir($path . "/" . $name ) && preg_match($match, $name))
				{
					if ($first) 
					{
						$first = false;
						$files = array();
					}
					$files[$name] = $this->traverseTree($path . "/" . $name, '/^[A-Z0-9][A-Z0-9][A-Z0-9]$/');
				}
			}
		    }
		    closedir($handle);
		}
	    }
	    return ($first ? substr($path, strlen($this->root)) : $files);
	}

	// Define the custom sort function for job names
	// name format is JJJJ_sss.lst
	// so we want to sort NUMERICALLY by JOB and SEQ numbers
	protected function jobNameSort($a,$b) 
	{	
		$ajob = explode(".", basename($a));
		$aseq = explode("_", $ajob[0]); 	// JOB in 0 and SEQ in 1

		$bjob = explode(".", basename($b));
		$bseq = explode("_", $bjob[0]);
		
		if ($aseq[0] == $bseq[0])		// JOB number
			return ($aseq[1] - $bseq[1]);	// SEQ numbers
			
		if ($this->JobOrderAscending === true)	
			return ($aseq[0] - $bseq[0]);	// ascending
		else
			return ($bseq[0] - $aseq[0]);	// descending
	}

	protected function sortOutputTree( & $tree)
	{
	    ksort($tree);	// class level
	    foreach ($tree as $class => $types) 
	    {
		ksort($types);	// type level
		$tree[$class] = $types;
	    }
	} 
	function getJobs($class, $type, $age=0, $search = null)
	{
	    $files = array();
	    $path = $this->root ."/". $this->oTree[$class][$type];
	    $now = time();
	    if ($search == "")
		    $search = null;

	    if(is_dir($path))
	    {
		if($handle = opendir($path))
		{
		    while(($name = readdir($handle)) !== false)
			if( $name != "." && $name != ".." &&  ! is_dir($name )) 
			{
				// have a file so check if  modified within $age minutes and if search require (case insensitive)
				if (($age == 0 || ($now - filemtime($path."/".$name)) < ($age * 60)) &&
				    ($search === null || stripos( file_get_contents($path."/".$name), $search)))			
						$files[$name] = $name;
			}
		    closedir($handle);
		}
	    }
	    uksort($files, array("jobOutput", "jobNameSort"));
	    return  $files;
	}
	function getJobData($class, $type, $job)
	{	
		$name = $this->root ."/". $this->oTree[$class][$type] ."/". $job;

		if ( ! is_readable($name))
			return null;
		$jobData = array();
		$jobData["mtime"] = filemtime($name);
		$jobData["size"]  = filesize($name);
		if ($fp = @fopen($name, "r"))
		{
			$jobData["9k"]= fread($fp, 9000);	// > 133 * 66 lines
			if(preg_match('/\*\*\*\*[A-Z0-9].*ROOM.*[A-Z0-9]\*\*\*\*/', $jobData["9k"], $matches)) { 	# MVS
				$jobData["desc"] = substr($matches[0], 24, 31);
				$jobData["date"] = substr($matches[0], 67, 21);
			} elseif (preg_match('/\*{12} .*START .* \*\*/', $jobData["9k"], $matches)) {			# VSE
				$jobData["desc"] = substr($matches[0], 31, 10);
				$jobData["date"] = substr($matches[0], 76, 21);
			} elseif (preg_match(
				'/JOB ([A-Za-z0-9_-]+) \([0-9]+\) QUEUED TO [A-Z0-9]* ON ([0-9]+-[A-Z]{3}-[0-9]{4} [0-9]{2}:[0-9]{2}) BY /', 
					$jobData["9k"], $matches)) {	# VMS
				$jobData["desc"] = $matches[1];
				$jobData["date"] = $matches[2];
			}
			fclose($fp);
		}
		return $jobData;
	}
	function getJobFP($class, $type, $job)
	{
		$name = $this->root ."/". $this->oTree[$class][$type] ."/". $job;

		if ( ! is_readable($name))
			return null;
		return fopen($name, "r");
	}
	
	function delete($class, $type, $job)
	{
		$name = $this->root ."/". $this->oTree[$class][$type] ."/". $job;

		if ( ! is_readable($name))
			return null;
		return unlink($name);
	}
	
}
?>