<?php
// Create a 132*66 image of the first page with a black dot representing each character
// and green bars to show what the page would look like from a distance. 
// Image can thus be used an icon.
//
// NOTE: This programme relies on having php5-gd or php7.X-gd installed.
//
function_exists('imagecreate') or die("php-gd functions not available.");

require_once("../lib/common.php");
require_once("../lib/banner.php");
require_once("../lib/job_class.php");
require_once("../lib/colours.php");

$lines = 66;	// per page
$columns = 132;

if (isset($_REQUEST["text"]))
{
	$bar_colour_name = getBarColourName();		// default colour will be picked
	$mtime = 0;// epoch so we never expire text
	caching_headers("TEXT ".$_REQUEST["text"], $mtime);
	$pagetext = make_banner($_REQUEST["text"]);
	$filename="icon_text";
} 
else if (isset($_REQUEST["job"]))
{
	$authorised = check_auth_ok("output");

	if( ! $authorised ) {
		header('HTTP/1.0 401 Unauthorized');
		echo "<h1>You don't have sufficient privileges to access job icons.</h1>";
		exit(1);
	}
	
	list($class, $type, $job) = getRequest();
	$filename = "{$class}_{$type}_{$job}";
	$bar_colour_name = getBarColourName($class, $type);
	$mvsprint = new jobOutput($PRT_DIR);
	$jobData = $mvsprint->getJobData($class, $type, $job);
	$mtime = $jobData['mtime'];

	caching_headers("ICON $bar_colour_name /$class/$type/$job", $mtime);
	$pagetext = $jobData["9k"];
} else
	die("Nothing for icon to do");

$border = 2;	// around all edges

$font_height = 1; //imagefontheight($font);
$font_width = 1; //imagefontwidth($font);

$im =  imagecreate($columns * $font_width + $border * 2,  $lines  + $border * 2);
$blk = imagecreate($font_width, $font_height);
$greenbar = imagecreate($columns * $font_width , $font_height);
$tb_border = imagecreate($columns * $font_width + $border * 2 , $font_height);	/* top and bottom border */
$s_border = imagecreate($font_width, $font_height * $lines + $border * 2);	/* side border */

// White background, black blocks and mid green bars

imagecolorallocate($im, 255, 255, 255);
imagecolorallocate($blk, 0, 0, 0);
imagecolorallocate($greenbar, ...getBarColourRGB($bar_colour_name));
imagecolorallocate($tb_border, 128, 128, 128);
imagecolorallocate($s_border, 128, 128, 128);

for ($line = 1 ; $line < $lines; $line += 2) { // draw green bar on alternate rows
	imagecopy($im, $greenbar, $border, $line * $font_height + $border, 0, 0, $columns * $font_width , $font_height) or die("no green bar");	
}

/* draw page borders */
imagecopy($im, $tb_border, 0, 0, 0, 0, $columns * $font_width + $border * 2 , $font_height);
imagecopy($im, $tb_border, 0, $lines * $font_height + 2* $border - $font_height, 0, 0, $columns * $font_width + $border * 2 , $font_height);
imagecopy($im, $s_border, 0, 0, 0, 0,                                        $font_width, $font_height * $lines + $border * 2);
imagecopy($im, $s_border, $font_width * $columns + $border * 2 - 1, 0, 0, 0, $font_width, $font_height * $lines + $border * 2);

$pagelen = strlen($pagetext);
$line = 0;	// line on page
$col = 0;	// position in line
for ($i = 0 ; $i < $pagelen && $line < $lines; $i++) {	
	if ($pagetext[$i] == "\f") 	// done page
		break;	
	if ($pagetext[$i] == "\n") {	// next line
		$col = 0; 
		$line++;
		continue;	
	} else if ($pagetext[$i] > " " && $col < $columns)  {	// printable character so set pixel to dot if not overflowing line
		imagecopy($im, $blk, $col * $font_width + $border, $line * $font_height + $border, 0, 0, $font_width, $font_height) 
			or die("cannot insert pixel for dot");
	}
	$col++;	
}

// Output the image

header('Content-type: image/png');
header("Content-Disposition: inline; filename=\"{$filename}.png\"");
imagepng($im) or die("Cannot create image as PNG");
imagedestroy($im);

?>
