<?php
//
// printer index file
//
// this traverses the printer classes output tree, as written by hercprt, and displays a side (nav) bar showing the tree and 
// the actual print jobs (as icons or a table) to allow a user to click on a particular job to view it.
// Options are provided, and remembered via cookies, to control output selection.
//
require_once("../lib/common.php");
require_once("../lib/colours.php");

$authorised = check_auth_ok("output");

if( ! $authorised ) {
	header('HTTP/1.0 401 Unauthorized');
	echo "<h1>You don't have sufficient privileges to access the output subsystem.</h1>";
	exit(1);
}

if ( ! file_exists($PRT_DIR)){
	header('HTTP/1.0 503 Service Unavailable');
	echo "<h1>The printer service directory has not been configured on the server.</h1>";
	exit(1);
}

require_once("../lib/job_class.php");	
$mvsprint = new jobOutput($PRT_DIR);
list($class, $type, $job) = getRequest();

if(isset($_POST["Delete"])) {
	echo "Deleting $class/$type/$job";
	$mvsprint->delete($class, $type, $job);
}

?>
<!DOCTYPE html>
<html>
<head>
<meta http-equiv="X-UA-Compatible" content="IE=edge">
<title>
MVS Output Classes (<?php echo gethostname(); ?>)
</title>
<link rel="stylesheet" type="text/css" href="../lib/core.css" media="screen" >
</head>
<body>
<?php
do_header("MVS Output");
?>
<nav>
<?php show_nav("printer", ".."); ?>
<h3>Output</h3>
<?php
// Display file directory tree from top level down

$tree = $mvsprint->getClassTypeTree();

if ($class === null)
	$class = key($tree);		// force to first class
if ($type === null)
	$type = key($tree[$class]);	// force to first type


foreach ($tree as $c => $c_array)
{
	echo "$c<br>";
	foreach ($c_array as $t => $t_array)
		if ($class === $c && $type === $t)
			echo "&nbsp;&nbsp;&nbsp;$t<br>";
		else
			echo "&nbsp;&nbsp;&nbsp;<a href=\"".$_SERVER["PHP_SELF"]."?class=$c&type=$t\">$t</a><br>\n";
}

?>
</nav>
<section>
<table border="1" >
<tr>
<td>
<b>Search</b>
<form name="search">
<?php
$search = isset($_REQUEST['search']) ? $_REQUEST['search'] : false;
?>
<input type="text" name="search" value="<?php echo $search; ?>" maxlength="80" size="40" onchange="javascript: submit();">
<input type="hidden" name="class" value="<?php echo $class; ?>">
<input type="hidden" name="type" value="<?php echo $type; ?>">
</form>
</td>
<td>
<b>Newer than</b>
<form name="age">
<input type="text" name="age" value="<?php echo $MVSControl['age']; ?>" maxlength="4" size="4" onchange="javascript: submit();">mins.
<input type="hidden" name="class" value="<?php echo $class; ?>">
<input type="hidden" name="type" value="<?php echo $type; ?>">
</form>
</td>
<td>
<b>Display Style</b>
<form name="style">
<input type="radio" name="style" value="list" onclick="javascript: submit();"<?php if ($MVSControl['style'] == "list") echo " checked"; ?>>List<br>
<input type="radio" name="style" value="icon" onclick="javascript: submit();"<?php if ($MVSControl['style'] == "icon") echo " checked"; ?>>Icon
<input type="hidden" name="class" value="<?php echo $class; ?>">
<input type="hidden" name="type" value="<?php echo $type; ?>">
</form>
</td>
<td>
<b>Sort Order</b>
<form name="order">
<input type="radio" name="order" value="asc" onclick="javascript: submit();"<?php if ($MVSControl['order'] == "asc") echo " checked"; ?>>Oldest<br>
<input type="radio" name="order" value="desc" onclick="javascript: submit();"<?php if ($MVSControl['order'] == "desc") echo " checked"; ?>>Newest
<input type="hidden" name="class" value="<?php echo $class; ?>">
<input type="hidden" name="type" value="<?php echo $type; ?>">
</form>
</td>
<td>
<b>PDF page</b>
<form name="pagesize">
<select name="pagesize" onchange="javascript: submit();">
<?php
 foreach (array("a4", "a5", "a3", "letter", "legal") as $s) {
	 $selected = ($s === $MVSControl['pagesize']) ? "selected" : "";
	 echo "<option value=\"$s\" $selected >$s</option>";
 }
 ?>
 </select>
<input type="hidden" name="class" value="<?php echo $class; ?>">
<input type="hidden" name="type" value="<?php echo $type; ?>">
</form>
</td>
<td>
<b>Colour</b>
<form name="colour">
<select name="colour" onchange="javascript: submit();">
<?php
$bar_colour = getBarColourName($class, $type);
 foreach ($colours as $c=> $rgb) {
	 $selected = ($c === $bar_colour ? "selected" : "");
	 echo "<option value=\"$c\" $selected >$c</option>";
 }
 ?>
 </select>
<input type="hidden" name="class" value="<?php echo $class; ?>">
<input type="hidden" name="type" value="<?php echo $type; ?>">
</form>
</td>
<td>
<form name="reload">
<input type="button" name="refresh" value="Refresh Jobs" onclick="javascript: submit()">
<input type="hidden" name="class" value="<?php echo $class; ?>">
<input type="hidden" name="type" value="<?php echo $type; ?>">
</form>
</td>
</tr>
</table>
<?php
if ($MVSControl['order'] == "asc")
	$mvsprint->sortAscending();

$files = $mvsprint->getJobs($class, $type, $MVSControl['age'], $search);

if ( ! $files || empty($files)) {
	echo "<br>No job files to display.<br>Select an Output CLASS/TYPE from the navigation bar on the left or widen the selection critriea.";
} else {
	echo "<h3>Showing output for Class $class of Type $type</h3>";
	// Sort the filenames by name with JOB and SEQ sorted numerically!
	if ($MVSControl['style'] == "list"){
		echo "<table><tr><th>Name</th><th>Description</th><th>Date</th><th>Size</th></tr>";
		$line = 1;
		$bar_hex = getBarColourHex($bar_colour);
		foreach ($files as $file) {
			if ($line++ % 2)
				echo "<tr style=\"background-color:$bar_hex\">";
			else
				echo '<tr>';

			echo "<td><a href=\"listfile.php?class=$class&type=$type&job=$file&colour=$bar_colour\" target=\"$file\">$file</a></td>\n";
			$job_data = $mvsprint->getJobData($class, $type, $file);
			echo "<td>";
			echo isset($job_data["desc"]) ? $job_data["desc"] : "";
			echo"</td><td align=\"right\">";
			echo isset($job_data["date"]) ? $job_data["date"] : "";
			echo"</td><td align=\"right\">";
			echo isset($job_data["size"]) ? number_format($job_data["size"]) : "";
			echo "</td>";
?>
		<td><form method="post">
			<input type="submit" value="Delete" name="Delete" class="button"
			onClick="return confirm('Really delete <?php echo $file?> ?');">
			<input type="hidden" value="<?php echo $file;?>" name="job">
			<input type="hidden" value="<?php echo $class;?>" name="class">
			<input type="hidden" value="<?php echo $type;?>" name="type">
		</form></td>
<?php
			echo "</tr>\n";
		}
		echo "</table>";
	} else {			// icon [default]
		foreach ($files as $file) {
			echo "<a href=\"listfile.php?class=$class&type=$type&job=$file&colour=$bar_colour\" target=\"$file\">
			<img src=\"icon.php?class=$class&type=$type&job=$file&colour=$bar_colour\" alt=\"$file\" ></a>\n";
		}
	}
} 
?>
</section>
</body>
</html>

