<?php
//
// tape index file
//
//allow users to upload, download, mount and delete tape image files
//
require_once("../lib/common.php");
$authorised = check_auth_ok("tape");

if( ! $authorised ) {
	header('HTTP/1.0 401 Unauthorized');
	echo "<h1>You don't have sufficient privileges to access the tape subsystem.</h1>";
	exit(1);
}

$msg = "";

// first check if we're being called to perform a function
if(isset($_POST["Upload"])) {
	$uploadOk = 1;
	$target_dir = "$TAPE_DIR/";
	$fname = preg_replace('((^\.)|\/|(\.$))', '_', $_FILES["tapeToUpload"]["name"]);	// clean
	$target_file = $target_dir . basename($fname);

	// Check if file already exists
	if (file_exists($target_file)) {
		$msg = "Sorry, Tape file already exists.";
		$uploadOk = 0;
	}
	// Check file size
	if ($_FILES["tapeToUpload"]["size"] > 50*1024*1024) {
		$msg = "Sorry, your tape file is larger than 50M.";
		$uploadOk = 0;
	}

	// Check if $uploadOk is set to 0 by an error
	if ($uploadOk == 0) {
		$msg .= " Tape was not uploaded.";
	// if everything is ok, try to upload file
	} else {
		if (move_uploaded_file($_FILES["tapeToUpload"]["tmp_name"], $target_file)) {
			$msg = "The tape file ". basename( $fname). " has been uploaded.";
		} else {
			$msg = "Sorry, there was an error uploading your tape file.";
		}
	}
}
if(isset($_POST["Delete"])) {
	// clean file
	$fname = preg_replace('((^\.)|\/|(\.$))', '_', $_POST["file"]);	// clean
	if (unlink("$TAPE_DIR/$fname")) {
		$msg = "Successfully deleted $fname";
	} else {
		$msg = "Could not delete $fname";
	}
}
if(isset($_POST["Download"])) {
	// clean file
	$fname = preg_replace('((^\.)|\/|(\.$))', '_', $_POST["file"]);	// clean
	if (! file_exists("$TAPE_DIR/$fname")) {
		$msg = "Tape file not found $fname";
	} else {
		header('Content-Type: application/octet-stream');
		header("Content-Transfer-Encoding: Binary"); 
		header("Content-disposition: attachment; filename=\"$fname\""); 
		readfile("$TAPE_DIR/$fname");
		return 0;
	}
}
if (isset($_POST["Mount"])) {
	$url = $CONS_INPUT."/cgi-bin/tasks/syslog";
	// clean unit by checking aginst CUUU format
	$unit = $_POST["unit"];
	if ( ! preg_match('/^[A-Fa-f0-9]{3,4}$/', $unit)) {
		$msg = "Invalid unit passed to mount.";
	} else {
		// clean file
		$fname = preg_replace('((^\.)|\/|(\.$))', '_', $_POST["file"]);	// clean
		if ( ! file_exists($TAPE_DIR."/".$fname)) {
			$msg = "Sorry, Tape file does not exist.";
		} else {
			$myvars = 'command=' . urlencode("devinit $unit $TAPE_DIR/$fname") . '&send=send';

			$ch = curl_init( $url );
			curl_setopt( $ch, CURLOPT_POST, 1);
			curl_setopt( $ch, CURLOPT_POSTFIELDS, $myvars);
			curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, 1);
			curl_setopt( $ch, CURLOPT_HEADER, 0);
			curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1);

			$response = curl_exec( $ch );
			if ($response === false) {
				$msg = "Could not issue mount of $fname on $unit";
				http_response_code(503);
			}
			else {
				$msg = "Mount of $fname requested on $unit";
				http_response_code(200);
			}
		}
	}
}


?>
<!DOCTYPE html>
<html>
<head>
<meta http-equiv="X-UA-Compatible" content="IE=edge">
<title>
MVS Tape Library (<?php echo gethostname(); ?>)
</title>
<link rel="stylesheet" type="text/css" href="../lib/core.css" media="screen" >
</head>
<body>
<?php
do_header("MVS Tape Library");
?>
<nav>
<?php show_nav("tape", ".."); ?>
</nav>
<?php
if ( ! is_dir($TAPE_DIR)) {
	// directory does not exist. Just in case the final path has not been created try and silently create it
	@mkdir($TAPE_DIR);
	if ( ! is_dir($TAPE_DIR)) {
		echo "<section><br><b>Cannot access or create tape repository</b><br></section></body></html>";
		exit;
	}
}
?>
<section>
<h2><?php echo $msg?></h2>
<h3>Load Tape Into Library</h3>
<form method="post" enctype="multipart/form-data">
    Select Tape to upload:
    <input type="file" name="tapeToUpload" id="tapeToUpload">
    <input type="submit" value="Upload Tape" name="Upload">
</form>
<br>
<?php
	
$files = glob("$TAPE_DIR/*");
$dirlen = strlen("$TAPE_DIR/");

if ( ! $files || empty($files)) {
	echo "<br><b>No tapes files to display.</b><br>";
} else {
	echo "<h3>Currently Available Tapes</h3>";
	echo "<table><tr><th>Name</th><th>Size</th><th>Mount to Unit</th><th></th><th></th></tr>";
	foreach ($files as $file) {
		$fname = basename($file);
?>
		<tr class="tr_line"><td><?php echo $fname;?></td>
		<td align="right"><?php echo filesize($file);?></td>
		
		<td><form method="post">
			<input type="submit" value="Mount" name="Mount" class="button">
			<input type="text" size="4" maxlength="4" value="480" pattern="[A-Fa-f0-9]{3,4}" name="unit">
			<input type="hidden" value="<?php echo $fname;?>" name="file">
		</form></td>
				
		<td><form action="tapemap.php" method="get">
			<input type="submit" value="Map" name="Map" class="button">
			<input type="hidden" value="<?php echo $fname?>" name="tape">
		</form></td>
		
		<td><form method="post">
			<input type="submit" value="Download" name="Download" class="button">
			<input type="hidden" value="<?php echo $fname?>" name="file">
		</form></td>
		
		<td><form method="post">
			<input type="submit" value="Delete" name="Delete" class="button"
			onClick="return confirm('Really delete <?php echo $fname?> ?');">
			<input type="hidden" value="<?php echo $fname;?>" name="file">
		</form></td>
		</tr>
<?php
	}
	echo "</table>";
} 
?>
</section>
</body>
</html>

