<?php
//
// tape index file
//
//for a given tape file show the VOLSER and DSNs
//
require_once("../lib/common.php");
$authorised = check_auth_ok("tape");

if( ! $authorised ) {
	header('HTTP/1.0 401 Unauthorized');
	echo "<h1>You don't have sufficient privileges to access the tape subsystem.</h1>";
	exit(1);
}

$msg = "";
$error = 0;
// aws tape functions
define("AWSTAPE_FLAG1_NEWREC",   0x80);  // Start of new record
define("AWSTAPE_FLAG1_TAPEMARK", 0x40);  // Tape mark
define("AWSTAPE_FLAG1_ENDREC",   0x20);  // End of record

// EBCDIC -> ASCII Translation table
$ebcdic_to_ascii = array(
0x00, 0x01, 0x02, 0x03, 0xA6, 0x09, 0xA7, 0x7F, 0xA9, 0xB0, 0xB1, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
0x10, 0x11, 0x12, 0x13, 0xB2, 0xB4, 0x08, 0xB7, 0x18, 0x19, 0x1A, 0xB8, 0xBA, 0x1D, 0xBB, 0x1F,
0xBD, 0xC0, 0x1C, 0xC1, 0xC2, 0x0A, 0x17, 0x1B, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0x05, 0x06, 0x07,
0xC8, 0xC9, 0x16, 0xCB, 0xCC, 0x1E, 0xCD, 0x04, 0xCE, 0xD0, 0xD1, 0xD2, 0x14, 0x15, 0xD3, 0xFC,
0x20, 0xD4, 0x83, 0x84, 0x85, 0xA0, 0xD5, 0x86, 0x87, 0xA4, 0xD6, 0x2E, 0x3C, 0x28, 0x2B, 0xD7,
0x26, 0x82, 0x88, 0x89, 0x8A, 0xA1, 0x8C, 0x8B, 0x8D, 0xD8, 0x21, 0x24, 0x2A, 0x29, 0x3B, 0x5E,
0x2D, 0x2F, 0xD9, 0x8E, 0xDB, 0xDC, 0xDD, 0x8F, 0x80, 0xA5, 0x7C, 0x2C, 0x25, 0x5F, 0x3E, 0x3F,
0xDE, 0x90, 0xDF, 0xE0, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0x60, 0x3A, 0x23, 0x40, 0x27, 0x3D, 0x22,
0xE7, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0xAE, 0xAF, 0xE8, 0xE9, 0xEA, 0xEC,
0xF0, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0xF1, 0xF2, 0x91, 0xF3, 0x92, 0xF4,
0xF5, 0x7E, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0xAD, 0xA8, 0xF6, 0x5B, 0xF7, 0xF8,
0x9B, 0x9C, 0x9D, 0x9E, 0x9F, 0xB5, 0xB6, 0xAC, 0xAB, 0xB9, 0xAA, 0xB3, 0xBC, 0x5D, 0xBE, 0xBF,
0x7B, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0xCA, 0x93, 0x94, 0x95, 0xA2, 0xCF,
0x7D, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0xDA, 0x96, 0x81, 0x97, 0xA3, 0x98,
0x5C, 0xE1, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0xFD, 0xEB, 0x99, 0xED, 0xEE, 0xEF,
0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0xFE, 0xFB, 0x9A, 0xF9, 0xFA, 0xFF
);

// convert string from ebcdic to ascii
function e2a($str)
{	global $ebcdic_to_ascii;

	$ascii = "";
	$l = strlen($str);
	for ($i = 0 ; $i < $l ; $i++) {
		$ascii .= chr($ebcdic_to_ascii[ord($str[$i])]);
	}
	return $ascii;
}

// return decoded half word starting at position p in string str
function hword($str, $p)
{
	return (ord($str[$p + 1])<<8) + ord($str[$p]);
}

function byte($str, $p)
{
	return ord($str[$p]);
}

// read an aws block return the data if this is a data block or false on a tape mark
function read_aws_block($fp, $conv = true)
{
	$awsbuf = fread($fp, 6);
	if (strlen($awsbuf) != 6)
		return false;
	$blen = hword($awsbuf, 0);
	// $plen = hword($awsbuf, 2);
	$flags1 = byte($awsbuf, 4);
	// $flags2 = byte($awsbuf, 5);
	if ($blen === 0 && ($flags1 | AWSTAPE_FLAG1_TAPEMARK))		// tape mark
		return false;
	$block = $blen > 0 ? fread($fp, $blen) : "";
	if (strlen($block) != $blen)
		return false;
	return ($conv ? e2a($block) : $block);
}

// check valid aws tape by reading and verifying first 6 byte header
function is_aws_tape($fp)
{
	$awsbuf = fread($fp, 6);
	fseek($fp, 0, 0);		// rewind 
	if (strlen($awsbuf) != 6)	// too short
		return false;
	$blen = hword($awsbuf, 0);
	$plen = hword($awsbuf, 2);
	if ($plen != 0) return false;	// invalid previous block length
	$flags1 = byte($awsbuf, 4);
	$flags2 = byte($awsbuf, 5);
	if ($flags2 != 0) return false;	// none zero flag 2 (unused ? )
	return true;	// potentially valid
}

// echo str with spaces as &nbsp; so they are shown in the browser
function echo_with_spaces($str)
{
	echo str_replace(" ", "&nbsp;",$str);
}


// first check if we're being called to perform a function
if(isset($_REQUEST["tape"])) {
	$fname = basename(preg_replace('((^\.)|\/|(\.$))', '_', $_REQUEST["tape"]));	// clean
	$tape_file = "$TAPE_DIR/" . $fname;

	// Check if file already exists
	if ( ! file_exists($tape_file)) {
		$msg = "Tape does not exist: $tape_file";
		$error++;
	} else if ( ! $tapefp = fopen($tape_file, "rb")) {
		$msg = "Could not open tape: $fname $tape_file";
		$error++;
	} if ( ! is_aws_tape($tapefp)) {
		$msg = "Tape does not appear to be AWS format";
		$error++;
	}
} else {
	$msg = "No tape specified".
	$error++;
}
?>
<!DOCTYPE html>
<html>
<head>
<meta http-equiv="X-UA-Compatible" content="IE=edge">
<title>
MVS Tape Map (<?php echo gethostname(); ?>)
</title>
<link rel="stylesheet" type="text/css" href="../lib/core.css" media="screen" >
</head>
<body>
<?php
do_header("MVS Tape Map");
?>
<section>
<?php
if ($error) 
	echo "<h2>$msg</h2>";
else {
	echo "<h3>Tape Map for $fname&nbsp;&nbsp;<button onclick=\"goBack()\">Back</button></h3>";
	$fileno = 0;
	while ($buf = read_aws_block($tapefp)) {		// headers
		$fileno++;
		do {
			if (strncmp($buf, "VOL1", 4) === 0)
				echo "<b>VOLSER => ". substr($buf, 4, 6)."</b><br>";
			if (strncmp($buf, "HDR1", 4) === 0)
				echo "<b>Tape File Num => $fileno DSN => ". substr($buf, 4, 17)."</b><br>";
			echo_with_spaces($buf);
			echo "<br>\n";
		} while ($buf = read_aws_block($tapefp));

		$count = 0;
		while ($buf = read_aws_block($tapefp, false))	// data don't process records
			$count++;
		echo "&nbsp;&nbsp;$count data blocks<br>";
		
		while ($buf = read_aws_block($tapefp))		// trailers
			echo_with_spaces($buf."<br>");
		echo "<br>";
	}
}

?>
<button onclick="goBack()">Back</button>

<script>
function goBack() {
    window.history.back();
 }
</script> 
</section>
</body>
</html>